@@ -1,7 +1,25 @@
-mod binding_files;
+use std::io::Write;
+use std::path::{Path, PathBuf};
+use std::process::{Command, Stdio};
+use std::{env, fs};
+
+use anyhow::{anyhow, Context, Result};
+use lazy_static::lazy_static;
+use regex::{Regex, RegexBuilder};
+use semver::Version;
+
+use build_tables::build_tables;
+use grammar_files::path_in_ignore;
+use grammars::{InlinedProductionMap, LexicalGrammar, SyntaxGrammar};
+use parse_grammar::parse_grammar;
+use prepare_grammar::prepare_grammar;
+use render::render_c_code;
+use rules::AliasMap;
+
 mod build_tables;
 mod char_tree;
 mod dedup;
+mod grammar_files;
 mod grammars;
 mod nfa;
 mod node_types;
@@ -11,21 +29,6 @@ mod render;
 mod rules;
 mod tables;
 
-use self::build_tables::build_tables;
-use self::grammars::{InlinedProductionMap, LexicalGrammar, SyntaxGrammar};
-use self::parse_grammar::parse_grammar;
-use self::prepare_grammar::prepare_grammar;
-use self::render::render_c_code;
-use self::rules::AliasMap;
-use anyhow::{anyhow, Context, Result};
-use lazy_static::lazy_static;
-use regex::{Regex, RegexBuilder};
-use semver::Version;
-use std::io::Write;
-use std::path::{Path, PathBuf};
-use std::process::{Command, Stdio};
-use std::{env, fs};
-
 lazy_static! {
     static ref JSON_COMMENT_REGEX: Regex = RegexBuilder::new("^\\s*//.*")
         .multi_line(true)
@@ -38,33 +41,65 @@ struct GeneratedParser {
     node_types_json: String,
 }
 
+pub const ALLOC_HEADER: &str = include_str!("./templates/alloc.h");
+
 pub fn generate_parser_in_directory(
-    repo_path: &PathBuf,
+    repo_path: &Path,
     grammar_path: Option<&str>,
     abi_version: usize,
     generate_bindings: bool,
     report_symbol_name: Option<&str>,
     js_runtime: Option<&str>,
 ) -> Result<()> {
-    let src_path = repo_path.join("src");
-    let header_path = src_path.join("tree_sitter");
+    let mut repo_path = repo_path.to_owned();
+    let mut grammar_path = grammar_path;
+
+    // Populate a new empty grammar directory.
+    if let Some(path) = grammar_path {
+        let path = PathBuf::from(path);
+        if !path
+            .try_exists()
+            .with_context(|| "Some error with specified path")?
+        {
+            fs::create_dir_all(&path)?;
+            grammar_path = None;
+            repo_path = path;
+        }
+    }
+
+    if repo_path.is_dir() && !repo_path.join("grammar.js").exists() && !path_in_ignore(&repo_path) {
+        if let Some(dir_name) = repo_path
+            .file_name()
+            .map(|x| x.to_string_lossy().to_ascii_lowercase())
+        {
+            if let Some(language_name) = dir_name
+                .strip_prefix("tree-sitter-")
+                .or_else(|| Some(dir_name.as_ref()))
+            {
+                grammar_files::generate_grammar_files(&repo_path, language_name, false)?;
+            }
+        }
+    }
 
     // Read the grammar.json.
-    let grammar_json = match grammar_path {
-        Some(path) => load_grammar_file(path.as_ref(), js_runtime)?,
-        None => {
-            let grammar_js_path = grammar_path.map_or(repo_path.join("grammar.js"), |s| s.into());
-            load_grammar_file(&grammar_js_path, js_runtime)?
-        }
+    let grammar_json = if let Some(path) = grammar_path {
+        load_grammar_file(path.as_ref(), js_runtime)?
+    } else {
+        let grammar_js_path =
+            grammar_path.map_or(repo_path.join("grammar.js"), std::convert::Into::into);
+        load_grammar_file(&grammar_js_path, js_runtime)?
     };
 
+    let src_path = repo_path.join("src");
+    let header_path = src_path.join("tree_sitter");
+
     // Ensure that the output directories exist.
     fs::create_dir_all(&src_path)?;
     fs::create_dir_all(&header_path)?;
 
     if grammar_path.is_none() {
-        fs::write(&src_path.join("grammar.json"), &grammar_json)
-            .with_context(|| format!("Failed to write grammar.json to {:?}", src_path))?;
+        fs::write(src_path.join("grammar.json"), &grammar_json)
+            .with_context(|| format!("Failed to write grammar.json to {src_path:?}"))?;
     }
 
     // Parse and preprocess the grammar.
@@ -81,7 +116,7 @@ pub fn generate_parser_in_directory(
         &language_name,
         syntax_grammar,
         lexical_grammar,
-        inlines,
+        &inlines,
         simple_aliases,
         abi_version,
         report_symbol_name,
@@ -89,10 +124,12 @@ pub fn generate_parser_in_directory(
 
     write_file(&src_path.join("parser.c"), c_code)?;
     write_file(&src_path.join("node-types.json"), node_types_json)?;
+    write_file(&header_path.join("alloc.h"), ALLOC_HEADER)?;
+    write_file(&header_path.join("array.h"), tree_sitter::ARRAY_HEADER)?;
     write_file(&header_path.join("parser.h"), tree_sitter::PARSER_HEADER)?;
 
-    if generate_bindings {
-        binding_files::generate_binding_files(&repo_path, &language_name)?;
+    if !path_in_ignore(&repo_path) {
+        grammar_files::generate_grammar_files(&repo_path, &language_name, generate_bindings)?;
     }
 
     Ok(())
@@ -107,7 +144,7 @@ pub fn generate_parser_for_grammar(grammar_json: &str) -> Result<(String, String
         &input_grammar.name,
         syntax_grammar,
         lexical_grammar,
-        inlines,
+        &inlines,
         simple_aliases,
         tree_sitter::LANGUAGE_VERSION,
         None,
@@ -116,10 +153,10 @@ pub fn generate_parser_for_grammar(grammar_json: &str) -> Result<(String, String
 }
 
 fn generate_parser_for_grammar_with_opts(
-    name: &String,
+    name: &str,
     syntax_grammar: SyntaxGrammar,
     lexical_grammar: LexicalGrammar,
-    inlines: InlinedProductionMap,
+    inlines: &InlinedProductionMap,
     simple_aliases: AliasMap,
     abi_version: usize,
     report_symbol_name: Option<&str>,
@@ -137,7 +174,7 @@ fn generate_parser_for_grammar_with_opts(
         &lexical_grammar,
         &simple_aliases,
         &variable_info,
-        &inlines,
+        inlines,
         report_symbol_name,
     )?;
     let c_code = render_c_code(
@@ -169,10 +206,7 @@ pub fn load_grammar_file(grammar_path: &Path, js_runtime: Option<&str>) -> Resul
         Some("json") => {
             Ok(fs::read_to_string(grammar_path).with_context(|| "Failed to load grammar.json")?)
         }
-        _ => Err(anyhow!(
-            "Unknown grammar file extension: {:?}",
-            grammar_path
-        )),
+        _ => Err(anyhow!("Unknown grammar file extension: {grammar_path:?}",)),
     }
 }
 
@@ -212,13 +246,32 @@ fn load_js_grammar_file(grammar_path: &Path, js_runtime: Option<&str>) -> Result
         .with_context(|| "Failed to read output from node")?;
     match output.status.code() {
         None => panic!("Node process was killed"),
-        Some(0) => {}
-        Some(code) => return Err(anyhow!("Node process exited with status {}", code)),
+        Some(0) => {
+            let stdout =
+                String::from_utf8(output.stdout).with_context(|| "Got invalid UTF8 from node")?;
+
+            let mut grammar_json = &stdout[..];
+
+            if let Some(pos) = stdout.rfind('\n') {
+                // If there's a newline, split the last line from the rest of the output
+                let node_output = &stdout[..pos];
+                grammar_json = &stdout[pos + 1..];
+
+                let mut stdout = std::io::stdout().lock();
+                stdout.write_all(node_output.as_bytes())?;
+                stdout.write_all(b"\n")?;
+                stdout.flush()?;
+            }
+
+            Ok(serde_json::to_string_pretty(
+                &serde_json::from_str::<serde_json::Value>(grammar_json)
+                    .with_context(|| "Failed to parse grammar JSON")?,
+            )
+            .with_context(|| "Failed to serialize grammar JSON")?
+                + "\n")
+        }
+        Some(code) => Err(anyhow!("Node process exited with status {code}")),
     }
-    let mut result =
-        String::from_utf8(output.stdout).with_context(|| "Got invalid UTF8 from node")?;
-    result.push('\n');
-    Ok(result)
 }
 
 fn write_file(path: &Path, body: impl AsRef<[u8]>) -> Result<()> {
